//This code is written by Minghao Wei, 2nd June, 2016
//This is the test code for ADC16 API
//This code is also an example of how to use the ADC16 API

#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <sys/select.h>
#include <sys/time.h>

#include "adc16lib.h"

#define PC104BASE 0x80008000//PC104Base address for TS-7250-V2.
#define ADCBASE 0x100//default address, may be different depend on jumper state.

//The PC104BASE is depend on the SBC your are using.
//See your SBC manual for detail information about that.
//In this example, I'm using 16-bit PC104 I/O from my TS-7250-V2.

//JP1 off, JP2 off, ADCBASE address 0x100.
//JP1 on , JP2 off, ADCBASE address 0x120.
//JP1 off, JP2 on , ADCBASE address 0x140.
//JP1 on , JP2 on , ADCBASE address 0x160.

//You can find all information about register and address in TS-wiki.

volatile uint16_t *adc16_io = 0;//This pointer will be your access to all function.
int main(){
//set up the pointer to adc.
        adc16_io = adcSetup(ADCBASE, PC104BASE);
//set clock speed to 50000, bigger number means slower FIFO fill up.
        setClockSpeed(adc16_io, 50000);
//set interrupt level.
        setInterrupt(adc16_io, 128);
//set DAC 0-3 (A-D)
        setDAC(adc16_io, 0, 0x0fff);//DAC A -> 5V
        setDAC(adc16_io, 1, 0x0bfd);//DAC B -> 3.75V
        setDAC(adc16_io, 2, 0x07fe);//DAC C -> 2.5V
        setDAC(adc16_io, 3, 0x03ff);//DAC D -> 1.25V
//start state machine with 3 channel pair (0-2)
        adcStart(adc16_io, 2);
//read 100 sample from FIFO.
//example of using getFIFOcount(), getHeadChannel(), readFIFO().
        int i;
        for (i = 0; i < 100; i ++){//print 100 sample to console
                printf("FIFO count %d, head channel %d\n", getFIFOcount(adc16_io
), getHeadChannel(adc16_io));
                printf("%d:FIFO 0x%4x\n", i + 1, readFIFO(adc16_io));
        }
//stop state machine
        adcStop(adc16_io);
//clean FIFO, remember to stop state machine before using cleanFIFO
        cleanFIFO(adc16_io);
//print board information to the console
        getBoardInfor(adc16_io);
        return 1;
}
