//Written by Minghao Wei for Technologic Systems
//This code will prove a simple API for the user
//This code is for educational intent also

//Implementation Note:
//This code is for TS-ADC16 use only.
//This code will assume your SBC's PC104 is on.

//Please see ADC16 manual for more information about base address and register.
//FIFO can hold 512 sample, State machine will stop when FIFO overflows
//FIFO empties in 3 condition:
//	1. Exttrig started state machine
//	2. SBC read out all data from FIFO
//	3. System power loss.

//This API will prove some simple functions
//adcSetup(adc16_base, PC104BASE)
//adcStart(adc16_io, pairNum)
//adcStop(adc16_io)
//readFIFO(adc16_io)
//setClockSpeed(adc16_io, speed)
//cleanFIFO(adc16_io)
//setInterrupt(adc16_io, level)
//getCount(adc16_io, count)
//setDAC(adc16_io, dacNum, level)
//getBoardInfor(adc16_io)
//getFIFOcount(adc16_io, output)
//getHeadChannel(adc16_io)
//setDIGIO(adc16_io, output)
//getDIGIO(adc16_io, input)

//These functions is very base and should be easy to understand
//Depend on your need, once you understand this code
//Writting your own code should give you better effciency

//Please see our ADC16 manual for detail information about register

#include <stdio.h> 
#include <stdint.h>
#include <errno.h>
#include <sys/mman.h>

//Use to setup a pointer access to adc registers 
//input:adc16 base address (default is 100, see manual), pc104 address for your SBC
//output:a point to the start of adc register
volatile uint16_t* adcSetup(uint16_t adc16_base, uint32_t PC104BASE);

//start the state machine 
//input: pointer created in the adcSetup function
//pairNum: number of channel pair you want to be activate, 0 - 7
void adcStart(volatile uint16_t* adc16_io, int pairNum);

//stop the state machine 
//input: pointer created in the adcSetup funcion
void adcStop(volatile uint16_t* adc16_io);

//read one sample from the ADCFIFO 
uint16_t readFIFO(volatile uint16_t* adc16_io); 

//set the speed of the 24bit clock counter
//bigger number means slower FIFO fill-up
//32000000 = sample frequency in Hz 
void setClockSpeed(volatile uint16_t* adc16_io, uint32_t speed); 

//Clean the FIFO by reading everything is in there
//stop the state machine before using thie function
void cleanFIFO(volatile uint16_t* adc16_io); 
 
//Set the interrupt level
//level default half full, 256 samples
//input level should be between 0-512
void setInterrupt(volatile uint16_t* adc16_io, uint16_t level); 

//return counter value
//input should be 0-3
//counter0, counter1, counter2, counter3
uint16_t getCount(volatile uint16_t* adc16_io, int count); 

//set DAC output, input dacNum -> 0 = A, 1 = B, 2 = C, 3 = D
//level-> 0x0 - 0xfff, 0xfff for 5V
//MAX 5V output
int setDAC(volatile uint16_t* adc16_io, int dacNum, uint16_t level);

//Display ID, PLD Revision, jumper status of the board
void getBoardInfor(volatile uint16_t* adc16_io);

//return the current FIFO count 
uint16_t getFIFOcount(volatile uint16_t* adc16_io);

//return the channel number that is in the head of FIFO
uint8_t getHeadChannel(volatile uint16_t* adc16_io);

//set DIGIO output, only 1-bit data
//input can be (0 | 1) 
void setDIGIO(volatile uint16_t* adc16_io, int output);

//return DIGIO input data, input with respect to input data number
//input should be 0-3, 0->input 0 data, 1 -> input 1 data......
uint8_t getDIGIO(volatile uint16_t* adc16_io, int input); 
